#ifndef COMPARTIMENTOH
#define COMPARTIMENTOH

#include "definicao.h"
#include "proptecido.h"
#include "transportesangue.h"

using namespace std;

/*************************************************************************//**
* \class	CompTecido 
* \brief	Representacao dos compartimentos pequenos vasos
* 			e tecido
* \author	Cyro Albuquerque Neto
* \date		Criacao: 27/01/2009
* \date		Revisao: 27/01/2009
*****************************************************************************/

enum TipoTecido { 
    PELE=0,GORDURA,MUSCULO,OSSO,CEREBRO,PULMAO,CORACAO,VISCERA,MUSCULO_CABECA,MUSCULO_PESCOCO
};

class CompTecido {
public:
	// Construtor
	CompTecido() : temperatura(36.6) {
		pressao.SetGas(8000.0,6000.0);
		pressaoAnt.SetGas(8000.0,6000.0);
		//concTecido.SetGas(8.0,20.0);
		concPeqVasos.SetGas(8.0,20.0);
	}

	// Destrutor
	~CompTecido() {}

	// Propriedades do tecido
	void SetPropTecido(PropTecido *pTec, TipoTecido tipo) { 
		propTecido = pTec; 
		tipoTec = tipo;
	}
	void SetGerCalorEsp(double valor) { propTecido->gerCalor = valor; }
	double Densidade() const { return propTecido->densidade; }
	double CalorEsp() const { return propTecido->calorEsp; }
	double CondTerm() const { return propTecido->condTerm; }
	double VazSangueEsp() const { return propTecido->vazSangue; }
	double GerCalorEsp() { return CorrMetabolismo(propTecido->gerCalor); }
	//double GerCalor() { return propTecido->gerCalor*volTotal; }
	double QuocResp() const { return propTecido->quocResp; }
	double TempBasal() const { return propTecido->tempBasal; }

    Gas<double> DerivadaConcentracaoTecido(Gas<double> presAux) {
        Gas<double> derAux; //mol/(m3.Pa)
        derAux.SetO2(9.2928e-6);    
        if (tipoTec==MUSCULO) {
            double p50 = 133.32237*exp(0.098*temperatura+2.748);
            derAux.SetO2(9.2928e-6 
                    + 0.05982/4.0*68000.0/17600.0*5.3
                    *p50/pow(presAux.O2()+p50,2.0));
        }
        derAux.SetCO2(propTecido->inclinacaoCurvaCO2);
        return derAux;
    }

	// Numero do compartimento
	void    SetNumComp(IntPos numero) { numComp = numero; }
	IntPos  NumComp() const { return numComp; }

	// Volume
	void    ZeraVolTotal() { volTotal = 0.0; }
	void    SomaVolTotal(double numero) { volTotal += numero; }
	//void  MultiplicaVolTotal(double numero) { volTotal *= numero; }
	double  VolTotal() const { return volTotal; }
	double  VolPeqVasos() const { return volPeqVasos; }
	double  VolTecido() const { return volTecido; }

	// Temperatura
	void    ZeraTemperatura() { temperatura = 0.0; }
	void    SomaTempVol(double numero) { temperatura += numero; }
	void    DivideTempVolTotal () { temperatura /= volTotal; }
	double  Temperatura() const { return temperatura; }

	//Tipo do tecido
	TipoTecido NomeTecido() { return tipoTec; }

	// Vazao de Sangue
	// OK pra concorrencia
	double  VazSangue() const { return propTecido->vazSangue*volTotal; }

	// Geracao (+) ou consumo (-) dos gases
	double  GerO2() {
		return CorrMetabolismo((-1.0)*propTecido->gerCalor*volTotal
			   /(16.17+4.83*propTecido->quocResp)/22.4e3);
	}
	double  GerCO2() {
		return CorrMetabolismo(propTecido->quocResp
               *propTecido->gerCalor*volTotal
			   /(16.17+4.83*propTecido->quocResp)/22.26e3);
    }
	//void    SetGerGas(Gas<double> val) { gerGas = val; }
	double  GerGas(IntPos i) { 
		if (i==0) return GerO2();
		else return GerCO2();
	}

	// Pressao parcial dos gases
	void        SetPressao(Gas<double> val) { pressao = val; }
	Gas<double> Pressao() const { return pressao; }
	double      Pressao(IntPos i) const { return pressao(i); }
	void        SetPressaoAnt(Gas<double> val) { pressaoAnt = val; }
	Gas<double> PressaoAnt() const { return pressaoAnt; }
	double      PressaoAnt(IntPos i) const { return pressaoAnt(i); }
    void        AtualizaPressaoAnt() { pressaoAnt = pressao; }

	// Concentracao dos gases
	void        SetConcPeqVasos(Gas<double> val) { concPeqVasos = val; }
	Gas<double> ConcPeqVasos() const { return concPeqVasos; }
	double      ConcPeqVasos(IntPos i) const { return concPeqVasos(i); }
	//void SetConcTecido(Gas<double> val) { concTecido = val; }
	//Gas<double> ConcTecido() const { return concTecido; }
	//double ConcTecido(IntPos i) const { return concTecido(i); }

    // Atualiza concentrao dos gases
    void AtualizaConcentracao() {
            concPeqVasos = transSangue.CalcConcentracao(
                    pressao,temperatura);
    }

    // Atualiza presso dos gases
    void AtualizaPressao() {
            pressao = transSangue.CalcPressaoParcial(
                    concPeqVasos,temperatura);
    }

    // Derivada da concentrao em relao a presso
    Gas<double> DerivadaConcentracaoSangue() {
        return transSangue.CalcDerivadaConcentracao(
                pressao,temperatura);
    }

    void AtualizaVolume() {
        volPeqVasos = volTotal*propTecido->relVolPeqVasos;
        volTecido = volTotal - volPeqVasos;
    }

    // Sadas dos dados
    void SaidaDadosArquivo(ostream &arq) {
        arq << VazSangue()*60.0e6 << TAB;
        arq << temperatura << TAB;

        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            arq << pressao(ig)/133.32 << TAB;
            arq << pressaoAnt(ig)/133.32 << TAB;
            arq << concPeqVasos(ig) << TAB;
            if (ig==0) arq << GerGas(ig)*60.0*22.4e3 << TAB;
            else arq << GerGas(ig)*60.0*22.26e3 << TAB;
        }
        arq << transSangue.PH() << TAB;
        arq << transSangue.SaturacaoO2()*100.0 << TAB;
    }



private:
	/// Tipo do tecido
	TipoTecido tipoTec;

	/// Numero do compartimento dentro de cada compartimento
	IntPos numComp;

	/// Volume de tecido
	double volTecido;

	/// Volume de sangue nos pequenos vasos
	double volPeqVasos;

	/// Volume total
	double volTotal;

	/// Vazo de sangue
	//double vazSangue;

	/// Temperatura media dos compartimentos
	double temperatura;

	/// Concentracao dos gases nos pequenos vasos
	Gas<double> concPeqVasos;

	/// Concentracao dos gases no tecido
	//Gas<double> concTecido;

	/// Pressao parcial dos gases
	Gas<double> pressao;
	Gas<double> pressaoAnt;

	/// Geracao (+) ou consumo (-) dos gases
	//Gas<double> gerGas;

	/// Caso seja musculo
	bool ehMusculo;

	/// Propriedades do tecido
	PropTecido *propTecido;

    /// Propriedades do sangue
    TransporteSangue transSangue;

	// Corrige o metabolismo em funcao da temperatura
	double CorrMetabolismo(double metabolismo) {
		return metabolismo*pow(2.0,(temperatura-propTecido->tempBasal)/10.0);
		//return metabolismo;
	}

};

/*************************************************************************//**
* \class	CompSangue 
* \brief	Representacao dos compartimentos grandes vasos
* \author	Cyro Albuquerque Neto
* \date		Criacao: 27/01/2009
* \date		Revisao: 27/01/2009
*****************************************************************************/

class CompSangue {
public:
	// Construtor
	CompSangue() : temperatura(36.6) {
		pressao.SetGas(8000.0,6000.0);
		pressaoAnt.SetGas(8000.0,6000.0);
		concentracao.SetGas(8.0,20.0);
	}

	// Destrutor
	~CompSangue() {}
	
	// Volume
	void   SetVolume(double valor) { volume=valor; }
	double Volume() const { return volume; }

	// Vazao
	//void   SetVazao(double valor) { vazao=valor; }
	//double Vazao() const { return vazao; }
	
	// Temperatura
	void   SetTemperatura(double valor) { temperatura=valor; }
	void   SetTemperaturaAnt(double valor) { temperaturaAnt=valor; }
	double Temperatura() const { return temperatura; }
	double TemperaturaAnt() const { return temperaturaAnt; }
    void   AtualizaTemperaturaAnt() { temperaturaAnt = temperatura; }

	// Pressao parcial dos gases
	void        SetPressao(Gas<double> val) { pressao = val; }
	Gas<double> Pressao() const { return pressao; }
	double      Pressao(IntPos i) const { return pressao(i); }
	void        SetPressaoAnt(Gas<double> val) { pressaoAnt = val; }
	Gas<double> PressaoAnt() const { return pressaoAnt; }
	double      PressaoAnt(IntPos i) const { return pressaoAnt(i); }
    void        AtualizaPressaoAnt() { pressaoAnt = pressao; }

	// Concentracao dos gases
	void        SetConcentracao(Gas<double> val) { concentracao = val; }
	Gas<double> Concentracao() const { return concentracao; }
	double      Concentracao(IntPos i) const { return concentracao(i); }

    // Atualiza concentrao dos gases
    void AtualizaConcentracao() {
        concentracao = transSangue.CalcConcentracao(pressao,temperatura);
    }

    //Sem compPulmao
    // Atualiza presso dos gases
    void AtualizaPressao() {
        pressao = transSangue.CalcPressaoParcial(concentracao,temperatura);
    }

    // Derivada da concentrao em relao a presso
    Gas<double> DerivadaConcentracao() {
        return transSangue.CalcDerivadaConcentracao(pressao,temperatura);
    }

    // Sadas dos dados
    void SaidaDadosArquivo(ostream &arq) {
        arq << temperatura << TAB;
        arq << temperaturaAnt << TAB;
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            arq << pressao(ig)/133.32 << TAB;
            arq << pressaoAnt(ig)/133.32 << TAB;
            arq << concentracao(ig) << TAB;
        }
        arq << transSangue.PH() << TAB;
        arq << transSangue.SaturacaoO2()*100.0 << TAB;
    }




private:
	/// Volume de sangue
	double volume;

	/// Vazao de sangue
	//double vazao;

	/// Temperatura
	double temperatura;
	double temperaturaAnt;

	/// Concentracao dos gases
	Gas<double> concentracao;

	/// Pressao parcial dos gases
	Gas<double> pressao;
	Gas<double> pressaoAnt;

    /// Propriedades do sangue
    TransporteSangue transSangue;

};


/*****************************************************//**
* \class	CompPulmao 
* \brief	Representacao do pulmo
* \author	Cyro Albuquerque Neto
* \date		Criacao: 27/01/2009
* \date		Revisao: 27/01/2009
*********************************************************/

class CompPulmao {
public:
	// Construtor
	CompPulmao() { 
        volAlveolar = 2000.0e-6; //m3
        volCapPulmTotal = 100.0e-6; //m3
        volTecTotal = 481.0e-6; //m3
        shunt = 0.98;
        presAlveolar.SetGas(15000.0,6000.0); //m3
        presAlveolarAnt.SetGas(15000.0,6000.0); //m3
        presCapPulm.resize(NUM_CAP_PULM);
        presCapPulmAnt.resize(NUM_CAP_PULM);
        concCapPulm.resize(NUM_CAP_PULM);
        transSangue.resize(NUM_CAP_PULM);
        temperatura=36.8;
	    for (IntPos i=0; i<NUM_CAP_PULM; ++i) {
            presCapPulm[i].SetGas(12000.0,6000.0); //m3
            presCapPulmAnt[i].SetGas(12000.0,6000.0); //m3
            concCapPulm[i].SetGas(10.0,20.0);
        }
        coefDifTotal.SetGas(26.0/(22.4e3*133.32*60.0),
                            180.0/(22.26e3*133.32*60.0));
        temperatura=37.0;
    }

	// Destrutor
	~CompPulmao() {}

	// Numero de compartimentos capilares pulmonares
	IntPos NumCompCapPulm() const { return NUM_CAP_PULM; }

	// Volume
    void   SetVolAlveolar(double val) { volAlveolar = val; }
    double VolAlveolar() const { return volAlveolar; }
	void   SetVolCapPulmTotal(double val) { volCapPulmTotal = val; }
    double VolCapPulm() const { return volCapPulmTotal/NUM_CAP_PULM; }
    void   SetVolTecTotal(double val) { volTecTotal = val; }
    double VolTecido() const { return volTecTotal/NUM_CAP_PULM; }

    // Desvio
    double Shunt() const { return shunt; }

	// Pressao parcial dos gases
	void        SetPresAlveolar(Gas<double> val) { presAlveolar = val; }
	Gas<double> PresAlveolar() const { return presAlveolar; }
	double      PresAlveolar(IntPos i) const { return presAlveolar(i); }
	void        SetPresAlveolarAnt(Gas<double> val) { presAlveolarAnt = val; }
	Gas<double> PresAlveolarAnt() const { return presAlveolarAnt; }
	double      PresAlveolarAnt(IntPos i) const { return presAlveolarAnt(i); }

    void        AtualizaPressaoAnt() { 
        presCapPulmAnt = presCapPulm;
        presAlveolarAnt = presAlveolar; 
    }

	void SetPresCapPulm(IntPos nComp, Gas<double> val) { 
        presCapPulm[nComp] = val;
    }
	Gas<double> PresCapPulm(IntPos nComp) const { 
        return presCapPulm[nComp]; 
    }
	double PresCapPulm(IntPos nComp, IntPos nGas) const { 
        return presCapPulm[nComp].Em(nGas);
    }

	void SetPresCapPulmAnt(IntPos nComp, Gas<double> val) { 
        presCapPulmAnt[nComp] = val;
    }
	Gas<double> PresCapPulmAnt(IntPos nComp) const { 
        return presCapPulmAnt[nComp]; 
    }
	double PresCapPulmAnt(IntPos nComp, IntPos nGas) const { 
        return presCapPulmAnt[nComp].Em(nGas);
    }


	// Concentracao dos gases
	void SetConcCapPulm(IntPos nComp, Gas<double> val) { 
        concCapPulm[nComp] = val;
    }
	Gas<double> ConcCapPulm(IntPos nComp) const { 
        return concCapPulm[nComp]; 
    }
	double ConcCapPulm(IntPos nComp, IntPos nGas) const { 
        return concCapPulm[nComp].Em(nGas);
    }
	double ConcUltimoCapPulm(IntPos nGas) const { 
        return concCapPulm[NUM_CAP_PULM-1].Em(nGas); 
    }

    // Atualiza concentrao dos gases
    void AtualizaConcentracao() {
        for (IntPos k=0; k<NUM_CAP_PULM; ++k) {
            concCapPulm[k] = transSangue[k].CalcConcentracao(
                presCapPulm[k],temperatura);
        }
    }

	// Temperatura
	void   SetTemperatura(double valor) { temperatura=valor; }
	double Temperatura() const { return temperatura; }

    // Atualiza presso dos gases
    void AtualizaPressao(IntPos k) {
        presCapPulm[k] = transSangue[k].CalcPressaoParcial(
                concCapPulm[k],temperatura);
    }

    // Derivada da concentrao em relao a presso
    Gas<double> DerivadaConcentracaoSangue(IntPos k) {
        return transSangue[k].CalcDerivadaConcentracao(
                presCapPulm[k],temperatura);
    }

    // Coeficiente de difuso local
    double CoefDifusao(IntPos iGas) const {
        return coefDifTotal(iGas)/NUM_CAP_PULM;
    }
    // Coeficiente de difuso total
    double CoefDifusaoTotal(IntPos iGas) {
        return coefDifTotal(iGas);
    }

    // Sadas dos dados
    void SaidaDadosArquivo(ostream &arq) {
        arq << temperatura << TAB;

        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            arq << presAlveolar(ig)/133.32 << TAB;
            arq << presAlveolarAnt(ig)/133.32 << TAB;
        }

        for (IntPos k=0; k<NUM_CAP_PULM; ++k) {
            for (IntPos ig=0; ig<NUM_GAS; ++ig) {
                arq << presCapPulm[k].Em(ig)/133.32 << TAB;
                arq << presCapPulmAnt[k].Em(ig)/133.32 << TAB;
                arq << concCapPulm[k].Em(ig) << TAB;
            }
            arq << transSangue[k].PH() << TAB;
            arq << transSangue[k].SaturacaoO2()*100.0 << TAB;
        }
    }


private:
    /// Nmero de capilares pulmonares
    static IntPos const NUM_CAP_PULM = 20;

    /// Volume alveolar
    double volAlveolar;

    /// Volume de sangue em todos compartimentos capilares pulmonares
    double volCapPulmTotal;

    /// Volume de tecido pulmonar
    double volTecTotal;

    /// Shunt
    double shunt;

    /// Temperatura
    double temperatura;

	/// Presso parcial alveolar dos gases
	Gas<double> presAlveolar;
	Gas<double> presAlveolarAnt;
    
    /// Presso parcial dos capilar pulmonares
	vector< Gas<double> > presCapPulm;
	vector< Gas<double> > presCapPulmAnt;

	/// Concentrao dos capilares pulmonares
	vector< Gas<double> > concCapPulm;

    /// Coeficiente de difuso total
    Gas<double> coefDifTotal;

    /// Propriedades do sangue
    vector< TransporteSangue > transSangue;

};

#endif
