#include "corpo.h"
#include <boost/thread/barrier.hpp>
#include <boost/ref.hpp>
#include "ThreadPassoCalculo.h"
#include "ControladorDaSimulacao.h"
#include <ctime>

extern bool regimePermanente;
extern double passoTempo;
extern double tempoSimulacao;
extern double relaxaPres;
extern ControladorDaSimulacao controladorDaSimulacao;
extern double tempoTotCalcPre;
extern double geracaoCalorMusculo;
extern double tempoTotCalc;
extern double tempoTotResto;

////////////////////////////// PUBLIC ////////////////////////////////////////

/*****************************************************************************
* Construtor
*****************************************************************************/
Corpo::Corpo()
{

	//Dados do ambiente
    ambiente.SetPropriedade(28.0,0.37,101325.0);

    LePropriedadeTecido();

    //Vetor com endereco dos segmentos
    lSegmento.push_back(&cabeca);
    lSegmento.push_back(&pescoco);
    lSegmento.push_back(&tronco);
    lSegmento.push_back(&bracoEsq);
    lSegmento.push_back(&antebracoEsq);
    lSegmento.push_back(&maoEsq);
    lSegmento.push_back(&bracoDir);
    lSegmento.push_back(&antebracoDir);
    lSegmento.push_back(&maoDir);
    lSegmento.push_back(&coxaEsq);
    lSegmento.push_back(&pernaEsq);
    lSegmento.push_back(&peEsq);
    lSegmento.push_back(&coxaDir);
    lSegmento.push_back(&pernaDir);
    lSegmento.push_back(&peDir);

    lSegmentoSaida.push_back(&cabeca);
    lSegmentoSaida.push_back(&pescoco);
    lSegmentoSaida.push_back(&tronco);
    lSegmentoSaida.push_back(&bracoEsq);
    lSegmentoSaida.push_back(&antebracoEsq);
    lSegmentoSaida.push_back(&maoEsq);
    lSegmentoSaida.push_back(&bracoDir);
    lSegmentoSaida.push_back(&antebracoDir);
    lSegmentoSaida.push_back(&maoDir);
    lSegmentoSaida.push_back(&coxaEsq);
    lSegmentoSaida.push_back(&pernaEsq);
    lSegmentoSaida.push_back(&peEsq);
    lSegmentoSaida.push_back(&coxaDir);
    lSegmentoSaida.push_back(&pernaDir);
    lSegmentoSaida.push_back(&peDir);

    corrResistTemp = 1.1;
    corrResistEvap = 1.1;

    cabeca.setNome("Cabeca");
    //cabeca.setEhEspelhado(false);

    pescoco.setNome("Pescoco");
    //pescoco.setEhEspelhado(false);

    tronco.setNome("Tronco");
    //tronco.setEhEspelhado(false);


    antebracoEsq.setNome("AnteBracoEsq");
    //antebracoEsq.setEhEspelhado(true);

    bracoEsq.setNome("BracoEsq");
    //bracoEsq.setEhEspelhado(true);

    maoEsq.setNome("MaoEsq");
    //maoEsq.setEhEspelhado(true);

    antebracoDir.setNome("AnteBracoDir");
    bracoDir.setNome("bracoDir");
    maoDir.setNome("MaoDir");

    coxaEsq.setNome("CoxaEsq");
    //coxaEsq.setEhEspelhado(true);

    pernaEsq.setNome("PernaEsq");
    //pernaEsq.setEhEspelhado(true);

    peEsq.setNome("PeEsq");
   // peEsq.setEhEspelhado(true);



    coxaDir.setNome("CoxaDir");
    pernaDir.setNome("PernaDir");
    peDir.setNome("PeDir");


    CriaSegmento(cabeca,"prop/cabeca.segmento");
    CriaSegmento(pescoco,"prop/pescoco.segmento");
    CriaSegmento(tronco,"prop/tronco.segmento");
    CriaSegmento(bracoEsq,"prop/braco.segmento");
    CriaSegmento(bracoDir,"prop/braco.segmento");
    CriaSegmento(antebracoEsq,"prop/antebraco.segmento");
    CriaSegmento(antebracoDir,"prop/antebraco.segmento");
    CriaSegmento(maoEsq,"prop/mao.segmento");
    CriaSegmento(maoDir,"prop/mao.segmento");
    CriaSegmento(coxaEsq,"prop/coxa.segmento");
    CriaSegmento(coxaDir,"prop/coxa.segmento");
    CriaSegmento(pernaEsq,"prop/perna.segmento");
    CriaSegmento(pernaDir,"prop/perna.segmento");
    CriaSegmento(peEsq,"prop/pe.segmento");
    CriaSegmento(peDir,"prop/pe.segmento");

    for (IntPos i=0; i<lSegmento.size(); ++i) {
        lSegmento[i]->SetAmbiente(&ambiente);
    }

    //ventilacao = 4200.0e-6/60.0;
    Segmento::ventilacao = 5000.0e-6/60.0;

    //Atualiza fracao dos gases inspirados
    tronco.AjustaFracaoInspirada(0.2094,0.0);

    for (IntPos i=0; i<lSegmento.size(); ++i) {
        //lSegmento[i]->SetSupMolhada(0.43);    //35 C
        //lSegmento[i]->SetSupMolhada(0.0915);  //30 C
        lSegmento[i]->SetSupMolhada(0.18);  //30 C
    }

    //caso segmento � tronco
    tronco.SetEhTronco(true);
}   


////////////////////////////// PRIVATE ///////////////////////////////////////

/*****************************************************************************
* Leitura das propriedades dos tecidos
*****************************************************************************/
void Corpo::LePropriedadeTecido()
{
    //Cria lista com endere�o das propriedades dos tecidos
    //Deve estar na ordem que os tecidos aparecem no arquivo
    //No caso do m�sculo, s� ser� lido o basal, depois m�sculos de cada
    //segmento recebem esse valor
    vector<PropTecido*> lTecido;

    //Associa lista de endere�os das propriedades a propriedade
    lTecido.push_back(&pele);
    lTecido.push_back(&gordura);
    lTecido.push_back(&muscBasal);
    lTecido.push_back(&osso);
    lTecido.push_back(&cerebro);
    lTecido.push_back(&pulmao);
    lTecido.push_back(&coracao);
    lTecido.push_back(&viscera);

    //N�mero de propriedades dos tecidos
    IntPos const NUM_PROPRIEDADE = 9;
    
    //Variaveis usadas na leitura das propriedades dos tecidos
    vector<double> lPropriedade(NUM_PROPRIEDADE,1.0); 
    
    //N�mero de strings no cabe�alho do arquivo de propriedades dos tecidos
    IntPos const NUM_STRING_CABECALHO = 19;

    //Abre arquivo com propriedades dos tecidos
    ifstream arqTecido("prop/tecido.dat");

    //Vari�vel para jogar fora string lido do arquivo
    string lixo;

    //Pula o cabe�alho
    for (IntPos i=0; i<NUM_STRING_CABECALHO; ++i) arqTecido >> lixo;
    
    //L� as propriedades
    for (IntPos i=0; i<lTecido.size(); ++i) {
        arqTecido >> lixo;
        for (IntPos j=0; j<NUM_PROPRIEDADE; ++j) {
            arqTecido >> lPropriedade[j];
        } 
        lTecido[i]->Inicializa(lPropriedade);
    }

    //Os m�sculos dos segmentos recebem propriedades do m�sculo basal
    muscPescoco = muscTronco = muscAnteBraco = muscBraco = muscMao 
        = muscCoxa = muscPerna = muscPe = muscCabeca = muscBasal;
    
    //Fecha arquivo
    arqTecido.close();
}

/*****************************************************************************
* Leitura das caracter�sticas de um segmento
*****************************************************************************/
void Corpo::CriaSegmento(Segmento& seg, const char* nomeArq)
{

    //Abre arquivo com caracter�sticas do segmento
    ifstream arquivo(nomeArq);

    //Vari�vel para jogar fora string lido do arquivo
    string lixo = "";
    
    //Vari�vel usada como auxiliar para a leitura do arquivo
    string aux = "";

    //essa parte deve sair quando ao inves de nDim e sistCord 
    //so tiver geomSegmento
    IntPos nDim=2;
    SistCoord sistCoord;

    //Tipo de geometria
    arquivo >> lixo;
    arquivo >> aux;
    if (aux == "RADIAL") {
        seg.SetGeometria(RADIAL); 
        nDim = 1;
        sistCoord = CILINDRICO;
    }
    else if (aux == "AXIAL") {
        seg.SetGeometria(AXIAL); 
        nDim = 2;
        sistCoord = CIL_RZ;
    }

    else if (aux == "POLAR") {
        seg.SetGeometria(POLAR); 
        nDim = 2;
        sistCoord = CILINDRICO;
    }
    else {
        seg.SetGeometria(RETANGULAR); 
        nDim = 2;
        sistCoord = CARTESIANO;
    }
    
    //Criacao das camadas
    vector<ListaCamada> *camadas = new vector<ListaCamada>(nDim);

    //Inclusao de camadas da direcao 1
    //N�mero de camadas
    IntPos numCamadasDir1=0;
    arquivo >> lixo;
    arquivo >> numCamadasDir1;
    //Limite e n�mero de c�lulas de cada camada
    double limite=0.0, numCelula=0.0;
    arquivo >> lixo;
    for (IntPos i=0; i<numCamadasDir1; ++i) {
        arquivo >> limite;
        arquivo >> numCelula;
        camadas->at(0).NovaCamada(limite,numCelula);
    }
    double tamExtDir1 = limite;

    //Inclusao de camadas da direcao 2
    //N�mero de camadas
    IntPos numCamadasDir2=0;
    arquivo >> lixo;
    arquivo >> numCamadasDir2;
    //Limite e n�mero de c�lulas de cada camada
    arquivo >> lixo;
    for (IntPos i=0; i<numCamadasDir2; ++i) {
        arquivo >> limite;
        arquivo >> numCelula;
        if (seg.Geometria()==POLAR) {
            camadas->at(1).NovaCamada(2.0*M_PI*limite/360.0,numCelula);
        }
        else camadas->at(1).NovaCamada(limite,numCelula); 
    }
    double tamExtDir2 = 0.0;
    if (numCamadasDir2!=0) {
        tamExtDir2 = limite;
    }

    //Comprimento, z caso radial, polar e retangular.
    //nao usado caso axial
    double comprimento;
    arquivo >> lixo;
    arquivo >> comprimento;

    //Geracao das camadas
    for (IntPos i=0; i<nDim; i++) camadas->at(i).GeraCamadas();

    //Plotagem das camadas
    for (IntPos i=0; i<nDim; i++) camadas->at(i).PlotaCamada();
    
    //Inicializacao do segmento
    seg.SetForma(nDim,sistCoord);

    //Tamanho externo
    if (seg.Geometria()==RADIAL || seg.Geometria()==POLAR) {
        seg.SetTamanhoExterno(tamExtDir1,2.0*M_PI,comprimento);
    }
    else if (seg.Geometria()==AXIAL) {
        seg.SetTamanhoExterno(tamExtDir1,tamExtDir2,2.0*M_PI);
    }
    else /*if (seg.Geometria()==RETANGULAR)*/ {
        seg.SetTamanhoExterno(tamExtDir1,tamExtDir2,comprimento);
    }

    //N�mero de tecidos
    IntPos numTecidos=0;
    arquivo >> lixo;
    arquivo >> numTecidos;

    //Cria compartimentos teciduais
    arquivo >> lixo;
    for (IntPos i=0; i<numTecidos; ++i) {
        arquivo >> aux;
        if (aux == "PELE") seg.NovoCompTecido(&pele,PELE);
        else if (aux == "GORDURA") seg.NovoCompTecido(&gordura,GORDURA);
        else if (aux == "OSSO") seg.NovoCompTecido(&osso,OSSO);
        else if (aux == "CEREBRO") seg.NovoCompTecido(&cerebro,CEREBRO);
        else if (aux == "PULMAO") seg.NovoCompTecido(&pulmao,PULMAO);
        else if (aux == "VISCERA") seg.NovoCompTecido(&viscera,VISCERA);
        else if (aux == "CORACAO") seg.NovoCompTecido(&coracao,CORACAO);
        else if (aux == "MUSCULO_CABECA") {
            seg.NovoCompTecido(&muscCabeca,MUSCULO);
        }
        else if (aux == "MUSCULO_PESCOCO") {
            seg.NovoCompTecido(&muscPescoco,MUSCULO);
        }
        else if (aux == "MUSCULO_TRONCO") {
            seg.NovoCompTecido(&muscTronco,MUSCULO);
        }
        else if (aux == "MUSCULO_ANTEBRACO") {
            seg.NovoCompTecido(&muscAnteBraco,MUSCULO);
        }
        else if (aux == "MUSCULO_BRACO") {
            seg.NovoCompTecido(&muscBraco,MUSCULO);
        }
        else if (aux == "MUSCULO_MAO") {
            seg.NovoCompTecido(&muscMao,MUSCULO);
        }
        else if (aux == "MUSCULO_COXA") {
            seg.NovoCompTecido(&muscCoxa,MUSCULO);
        }
        else if (aux == "MUSCULO_PERNA") {
            seg.NovoCompTecido(&muscPerna,MUSCULO);
        }
        else /*if (aux == "MUSCULO_PE")*/ {
            seg.NovoCompTecido(&muscPe,MUSCULO);
        }
    }

    //Local dos compartimentos teciduais
    Matriz<CompTecido*> matCompTec;
    if (nDim==1) {
        matCompTec.SetDimensao(camadas->at(0).Tam(),1,1,
                               seg.PtCompTecido(2));
    }
    else /*if (nDim==2)*/ {
        matCompTec.SetDimensao(camadas->at(0).Tam(), 
                               camadas->at(1).Tam(),1,
                               seg.PtCompTecido(2));
    }

    IntPos idTecido=2;
    arquivo >> lixo;
    //corre��o para que seja feita a leitura da distribui��o de tecidos
    //no caso radial
    if (numCamadasDir2==0) numCamadasDir2 = 1;  
    //Leitura da matriz
    for (IntPos j=0; j<numCamadasDir2; ++j) {
        for (IntPos i=0; i<numCamadasDir1; ++i) {
            arquivo >> idTecido; 
            matCompTec(i,j,0) = seg.PtCompTecido(idTecido);
        }
    }

    //Associa matriz de compartimento tecidual
    seg.Inicializa(*camadas,matCompTec);

    //Resistencia entra pele e ambiente
    double resConv=0.0;
    double resRad=0.0;
    double resEvap=0.0;
    arquivo >> lixo;
    arquivo >> resConv; 
    arquivo >> resRad;  
    arquivo >> resEvap; 
    seg.SetResistTemperatura(corrResistTemp*(resConv+resRad));
    seg.SetResistEvaporacao(corrResistEvap*resEvap);
    seg.SetSupMolhada(0.06);

    //Coeficiente de transferencia de calor entre 
    //compartimento arterial e venoso
    double coefArtVen=0.0;
    arquivo >> lixo;
    arquivo >> coefArtVen;
    seg.SetCoefArtVen(coefArtVen);

    //Volume do compartimento arterial e venoso
    double volArt=0.0, volVen=0.0;
    arquivo >> lixo;
    arquivo >> volArt;
    arquivo >> volVen;
    seg.SetVolArtVen(volArt*1.0e-6,volVen*1.0e-6);

    //Apaga camadas
    delete camadas;

    //Fecha arquivo
    arquivo.close();
}


void Corpo::inicializaSaidas(vector<ofstream*>& saidaTrans)
{
	// Segmentos
	for (IntPos i=0; i<lSegmentoSaida.size(); ++i) {
		// tempo
		*saidaTrans[i] << "#hr\tmin\tseg\t";
		// vaz�es
		*saidaTrans[i] << "Vzant\tVzpos\tVzsv\tsupMolhada\tTsupMed\tQamb\t";
		*saidaTrans[i] << "CO2arEnt\tCCO2arEnt\tCO2veEnt\tCCO2veEnt\t";
		// arterial
		*saidaTrans[i] << "Tar\tTar0\t";
		*saidaTrans[i] << "PO2ar\tPO2ar0\tCO2ar\t";
		*saidaTrans[i] << "PCO2ar\tPCO2ar0\tCCO2ar\t";
		*saidaTrans[i] << "pHar\tSO2ar\t";
		// venoso
		*saidaTrans[i] << "Tve\tTve0\t";
		*saidaTrans[i] << "PO2ve\tPO2ve0\tCO2ve\t";
		*saidaTrans[i] << "PCO2ve\tPCO2ve0\tCCO2ve\t";
		*saidaTrans[i] << "pHve\tSO2ve\t";
		// tecidos
		for (IntPos j=0; j<lSegmentoSaida[i]->QuantCompTecido(); ++j) {
			*saidaTrans[i] << "Vzbl_t" << j << TAB;
			*saidaTrans[i] << "T_t" << j << TAB;
			*saidaTrans[i] << "PO2_t" << j << TAB;
			*saidaTrans[i] << "PO2ant_t" << j << TAB;
			*saidaTrans[i] << "CO2_t" << j << TAB;
			*saidaTrans[i] << "VO2_t" << j << TAB;
			*saidaTrans[i] << "PCO2_t" << j << TAB;
			*saidaTrans[i] << "PCO2ant_t" << j << TAB;
			*saidaTrans[i] << "CCO2_t" << j << TAB;
			*saidaTrans[i] << "VCO2_t" << j << TAB;
			*saidaTrans[i] << "pHsv_t" << j << TAB;
			*saidaTrans[i] << "SO2sv_t" << j << TAB;
			if (lSegmentoSaida[i]->EhPulmao(j)) {
				*saidaTrans[i] << "Tpulm\tPO2A\tPO2Aant\tPCO2A\tPCO2Aant\t";
				for (IntPos k=1; k<=20; ++k) {
					*saidaTrans[i] << "PO2cp" << k << TAB;
					*saidaTrans[i] << "PO2cpant" << k << TAB;
					*saidaTrans[i] << "CO2cp" << k << TAB;
					*saidaTrans[i] << "PCO2cp" << k << TAB;
					*saidaTrans[i] << "PCO2cpant" << k << TAB;
					*saidaTrans[i] << "CCO2cp" << k << TAB;
					*saidaTrans[i] << "pHcp" << k << TAB;
					*saidaTrans[i] << "SO2cp" << k << TAB;
				}
			}
		}

		*saidaTrans[i] << endl;
	}
}

/*****************************************************************************
* Calculo
*****************************************************************************/
void Corpo::Calcula()
{
double tempoTotOut = 0.0;
    //saida dos volumes
    /*cout << "Volumes: " << endl;
    for (IntPos i=0; i<lSegmento.size(); ++i) {
        for (IntPos j=0; j<lSegmento[i]->QuantCompTecido(); ++j) {
            cout << lSegmento[i]->PtCompTecido(j)->VolTotal()*1.0e6 << " ";
        }
        cout << endl;
    }*/

    //muscBasal.gerCalor = 684.0;  //35 C
    //muscBasal.gerCalor = 3340.0;  //25 C  
    muscBasal.gerCalor = 1717.0;    //30 C  
    muscBasal.vazSangue =(muscBasal.gerCalor-684.0)*2.39e-7 + 543.0e-6; 
    muscPescoco = muscTronco = muscAnteBraco = muscBraco = muscMao = muscCoxa = muscPerna = muscPe = muscCabeca = muscBasal;
    //pele.vazSangue = 215.0e-6;    //25 C
    //pele.vazSangue = 500.0e-6;    //35 C

    IntPos maxIteracao = 200000;
    bool atingiuPrecisao = false;
    IntPos iteracao=0;

    double dTempoSaida = 60.0, tempoProximaSaida = 0.0;

    double calGerado=0.0, calPerdido=0.0;
    double tempPeleMedia=0.0, tempInterna=0.0;
    double calafrio = 0.0;
    
    double gerTotal = 85.0;  //sem considerar o calafrio

    regimePermanente = true;

    double tempoGastoParaConstruirThreads = 0.0;

    //ofstream saidaTrans("saida/transitorio.txt");
   
    // Inicializa��o dos arquivos de sa�da em RT
    ofstream saidaGlobal("saida/trans_global.txt");
    
    ofstream saidaCabeca("saida/trans_cabeca.txt");
    ofstream saidaPescoco("saida/trans_pescoco.txt");
    ofstream saidaTronco("saida/trans_tronco.txt");
    ofstream saidaBracoEsq("saida/trans_bracoEsq.txt");
    ofstream saidaAntebracoEsq("saida/trans_antebracoEsq.txt");
    ofstream saidaMaoEsq("saida/trans_maoEsq.txt");
    ofstream saidaBracoDir("saida/trans_bracoDir.txt");
    ofstream saidaAntebracoDir("saida/trans_antebracoDir.txt");
    ofstream saidaMaoDir("saida/trans_maoDir.txt");
    ofstream saidaCoxaEsq("saida/trans_coxaEsq.txt");
    ofstream saidaPernaEsq("saida/trans_pernaEsq.txt");
    ofstream saidaPeEsq("saida/trans_peEsq.txt");
    ofstream saidaCoxaDir("saida/trans_coxaDir.txt");
    ofstream saidaPernaDir("saida/trans_pernaDir.txt");
    ofstream saidaPeDir("saida/trans_peDir.txt");

    vector<ofstream*> saidaTrans;
    
    saidaTrans.push_back(&saidaCabeca);
    saidaTrans.push_back(&saidaPescoco);
    saidaTrans.push_back(&saidaTronco);
    saidaTrans.push_back(&saidaBracoEsq);
    saidaTrans.push_back(&saidaAntebracoEsq);
    saidaTrans.push_back(&saidaMaoEsq);
    saidaTrans.push_back(&saidaBracoDir);
    saidaTrans.push_back(&saidaAntebracoDir);
    saidaTrans.push_back(&saidaMaoDir);
    saidaTrans.push_back(&saidaCoxaEsq);
    saidaTrans.push_back(&saidaPernaEsq);
    saidaTrans.push_back(&saidaPeEsq);
    saidaTrans.push_back(&saidaCoxaDir);
    saidaTrans.push_back(&saidaPernaDir);
    saidaTrans.push_back(&saidaPeDir);

    // Cabe�alho

    // Global
    saidaGlobal << "#hr\tmin\tseg\t";
    saidaGlobal << "Tin\tTskMed\tTamb\t";
    saidaGlobal << endl;

    /*
    boost::thread threadInicializaSaidas(&Corpo::inicializaSaidas, this, boost::ref(saidaTrans) );
    threadInicializaSaidas.join();
    */

    inicializaSaidas(saidaTrans);


    //liberaCalculoTemperatura, esperaTerminoCalculoTemperatura


    int const numGrupoSeg = 4;

    vector<int> grupo1;
    vector<int> grupo2;
    vector<int> grupo3;
    vector<int> grupo4;

    grupo1.push_back(0);
    grupo1.push_back(1);
    grupo1.push_back(10);
    grupo1.push_back(13);
    grupo1.push_back(9);
    grupo1.push_back(12);

    grupo2.push_back(5);
    grupo2.push_back(11);
    grupo2.push_back(3);
    grupo2.push_back(4);

    grupo3.push_back(8);
    grupo3.push_back(14);
    grupo3.push_back(6);
    grupo3.push_back(7);

    grupo4.push_back(2);


    boost::barrier esperaTerminoCalculo(numGrupoSeg + 1);
    boost::barrier barreiraDefinicaoEscopoProximaIteracao(numGrupoSeg + 1);
    boost::barrier sincronizaFimExecucao(numGrupoSeg  + 1);

    ThreadPassoCalculo* tc[numGrupoSeg];

    vector< vector<int> > grupos;
    grupos.push_back(grupo1);
    grupos.push_back(grupo2);
    grupos.push_back(grupo3);
    grupos.push_back(grupo4);

    boost::thread_group threadsPassoCalculo;
    for (IntPos i=0; i<numGrupoSeg; ++i) {
    	tc[i] = new ThreadPassoCalculo(lSegmento, grupos[i], &barreiraDefinicaoEscopoProximaIteracao, &esperaTerminoCalculo, &sincronizaFimExecucao);
    	threadsPassoCalculo.add_thread(new boost::thread(boost::ref(*tc[i])));
    }

    for (double tempo=0.0; tempo<=tempoSimulacao; tempo+=passoTempo) {
       
    	 for (IntPos i=0; i<lSegmento.size(); ++i) {
    		 lSegmento[i]->setAtingiuPrecisaoCalcTemp(false);
    		 lSegmento[i]->setAtingiuPrecisaoCalcPressao(false);
    	 }

        if (tempo <= 60.0*60.0) {
            ambiente.SetTempOperativa(28.0);
            ambiente.SetUmidadeRel(0.37);
        }
        else if (tempo > 60.0*60.0 && tempo <= 180.0*60.0) {
            ambiente.SetTempOperativa(42.5);
            ambiente.SetUmidadeRel(0.28);
        }
        else {
            ambiente.SetTempOperativa(28.0);
            ambiente.SetUmidadeRel(0.37);
        }

        /*else if (tempo > 120.0*60.0 && tempo <= 180.0*60.0) {
            ambiente.SetTempOperativa(28.0);
            ambiente.SetUmidadeRel(0.37);
        }
        else if (tempo > 180.0*60.0 && tempo <= 240.0*60.0) {
            ambiente.SetTempOperativa(15.0);
            ambiente.SetUmidadeRel(0.45);
        }
        else {
            ambiente.SetTempOperativa(28.0);
            ambiente.SetUmidadeRel(0.37);
        }*/

        for (iteracao=0; iteracao<=maxIteracao; ++iteracao) {
            atingiuPrecisao = true;

            time_t tempoResto = time(NULL);

            //ajusta fator de relaxamento da press�o parcial
            /*if (!regimePermanente) {
                if (iteracao<2000) {
                    relaxaPres = 0.01*exp(log(100.0)/2000.0*iteracao);
                }
                else relaxaPres = 1.0;
            }
            else relaxaPres = 0.1;*/

            AjustaVazao();
            AjustaTemperatura();
            AjustaConcentracao();

            /*
            boost::thread threadAjustaTemp(&Corpo::AjustaTemperatura, this);
            boost::thread threadAjustaConc(&Corpo::AjustaConcentracao, this);
            threadAjustaTemp.join();
            threadAjustaConc.join();
            */
                
            //calcula calor total gerado
            calGerado=0.0;
            calPerdido=0.0;
            for (IntPos i=0; i<lSegmento.size(); ++i) {
            	calPerdido += lSegmento[i]->CalorProAmbiente();;
            	calGerado += lSegmento[i]->GerCalor();

            }
            calPerdido -= tronco.CalorRespiracao(ambiente.TempOperativa(),
                   ambiente.UmidadeRel());
                
            double volTotal=0.0;
            double volComp=0.0;
            tempPeleMedia=0.0;
            for (IntPos i=0; i<lSegmento.size(); ++i) {


            		volComp = lSegmento[i]->VolumeTecido(PELE);
            		volTotal += volComp;
            		tempPeleMedia += lSegmento[i]->TempCompTecido(PELE)*volComp;


            }
            tempPeleMedia/=volTotal;
            tempInterna = cabeca.TempCompTecido(CEREBRO);

            //Calafrio 
            //if (iteracao==0 && tempo==0.0) calafrio = 0.0;
            //else calafrio = Calafrio(tempPeleMedia,tempInterna,-calPerdido);
            calafrio = Calafrio(tempPeleMedia,tempInterna,-calPerdido);
            muscBasal.gerCalor = 684.0 + (gerTotal-69.62)/26330.0e-6 + calafrio;

            muscBasal.vazSangue =(muscBasal.gerCalor-684.0)*2.39e-7 + 543.0e-6; 
            muscPescoco = muscTronco = muscAnteBraco = muscBraco = muscMao 
                = muscCoxa = muscPerna = muscPe = muscCabeca = muscBasal;
            geracaoCalorMusculo = muscBasal.gerCalor;

            //vazao de sangue na pele
            pele.vazSangue = VazaoPele(tempPeleMedia,tempInterna);

            //Sudorese

            for (IntPos i=0; i<lSegmento.size(); ++i) {
           		lSegmento[i]->AjustaSupMolhada(tempPeleMedia,tempInterna);
            }


//            time_t tempoThreadSupMolhada = time(NULL);


            /*
            boost::thread_group threadsParaAjustarSupMolhadaDosSegmentos;

            for ( IntPos i = 0; i < lSegmento.size(); ++i)
            	threadsParaAjustarSupMolhadaDosSegmentos.add_thread(new boost::thread(&Segmento::AjustaSupMolhada,
            			lSegmento[i], tempPeleMedia, tempInterna));

            //tempoGastoParaConstruirThreads += (time(NULL) - tempoThreadSupMolhada);

            threadsParaAjustarSupMolhadaDosSegmentos.join_all();
*/

            //Respira��o
            Segmento::ventilacao = (69.34*calGerado-1521.4)*1.0e-6/60.0;
            
            //Aqui ajusta calor perdido pela respira��o,
            //ajusta perdas no m�sculo do pesco�o e cabe�a
            //e no pulm�o
            AjustaCalorRespiracao();
/*
            for (IntPos i=0; i<lSegmento.size(); ++i) {
            	lSegmento[i]->PassoCalculoTemperatura();
            	if (!lSegmento[i]->getAtingiuPrecisaoCalcTemp())
            //	if (!lSegmento[i]->PassoCalculoTemperatura())
                    atingiuPrecisao=false;
            }
  */


            /*
            boost::thread_group threadsPassoCalculoTemperatura;*/
        /*    boost::thread_group threadsPassoCalculo;
                        for (IntPos i=0; i<lSegmento.size(); ++i) {
                        	threadsPassoCalculo.add_thread(new boost::thread(&Segmento::PassoCalculo, lSegmento[i]));
                        	lSegmento[i]->PassoCalculo();
                        }

                        threadsPassoCalculo.join_all();

                        */

            tempoTotResto += (time(NULL) - tempoResto);

            time_t tempoThreads = time(NULL);

            barreiraDefinicaoEscopoProximaIteracao.wait();

            esperaTerminoCalculo.wait();

            tempoTotCalc += (time(NULL) - tempoThreads);

            for (IntPos i=0; i<lSegmento.size(); ++i) {
            	//lSegmento[i]->PassoCalculo();

            	if (!lSegmento[i]->getAtingiuPrecisaoCalcTemp())
            		atingiuPrecisao=false;
            	if (!lSegmento[i]->getAtingiuPrecisaoCalcPressao())
            		atingiuPrecisao=false;
            }


/*            boost::thread_group threadsPassoCalculoTemperatura;
            for (IntPos i=0; i<lSegmento.size(); ++i) {
            	threadsPassoCalculoTemperatura.add_thread(new boost::thread(&Segmento::PassoCalculoTemperatura, lSegmento[i]));
            }

            threadsPassoCalculoTemperatura.join_all();
            */


            /*
            for (IntPos i=0; i<lSegmento.size(); ++i) {
            	tct[i]->permiteCalculo();
            }
            {
            	boost::mutex::scoped_lock lock(mutexControle);
            	liberaCalculoTemperatura.notify_all();
            }
            */



            /*
            <MEU>

            barreiraDefinicaoEscopoProximaIteracaoTemp.wait();

            esperaTerminoCalculoTemperatura.wait();

            tempoGastoParaConstruirThreads += (time(NULL) - tempoThreadTemp);

            for (IntPos i=0; i<lSegmento.size(); ++i) {
            	if (!lSegmento[i]->getAtingiuPrecisaoCalcTemp())
            		atingiuPrecisao=false;
            }




            */



//            muscPescoco.gerCalor = muscCabeca.gerCalor = muscBasal.gerCalor;
//            pulmao.gerCalor = 339.0;


            /*
            for (IntPos i=0; i<lSegmento.size(); ++i) {
                if (!lSegmento[i]->PassoCalculoPressao()) 
                    atingiuPrecisao=false;
            }
            */


            /*
            time_t tempoThreadPressao = time(NULL);
            boost::thread_group threadsPassoCalculoPressao;
            for (IntPos i=0; i<lSegmento.size(); ++i) {
            	threadsPassoCalculoPressao.add_thread(new boost::thread(&Segmento::PassoCalculoPressao, lSegmento[i]));
            }

            threadsPassoCalculoPressao.join_all();

			*/




            /*
             *
             <MEU>





            time_t tempoCalcPre = time(NULL);

            barreiraDefinicaoEscopoProximaIteracaoPressao.wait();

            esperaTerminoCalculoPressao.wait();

            tempoTotCalcPre += (time(NULL) - tempoCalcPre);

           // tempoGastoParaConstruirThreads += (time(NULL) - tempoThreadPressao);

            for (IntPos i=0; i<lSegmento.size(); ++i) {
            	if (!lSegmento[i]->getAtingiuPrecisaoCalcPressao())
            		atingiuPrecisao=false;
            }




*/




            if (atingiuPrecisao) {
                for (IntPos i=0; i<lSegmento.size(); ++i) {
               		lSegmento[i]->AtualizaValorAnt();
                }
                break;
            }
            if (iteracao==maxIteracao) {
               cout << "Atingiu limite de itera��o do c�lculo da condi��o inicial"
                    << endl; 
               exit(1);
            }

            if (iteracao%100==0) { 
                cout << iteracao 
                     << " | calor perdido/gerado: " << calPerdido 
                     << "/" << calGerado 
                     //<< " VA: " << Segmento::ventilacao*60.0e6 
                     << " Vsk: " << pele.vazSangue*1.0e6 
                     //<< " Tar: " << tronco.TempArterial()
                     << " w: " << pernaEsq.SupMolhada()
                     << " calaf: " << calafrio << endl;
                     //<< " Tar0: " << tronco.TempArterialAnt() << endl;
                     //<< " cal: " << calafrio << endl;
            }

            //exit(1);  //sai fora

            //if (tempo>=tempoProximaSaida) cout << "iteracao: " << iteracao;

        } //iteracao

        regimePermanente = false;

		// Saida dos dados
		if (tempo>=tempoProximaSaida) {

			time_t tempoOutput = time(NULL);

            //saida tela
            cout << "t = " << tempo/60.0 <<  " min" << endl;

            //saida dos dados globais
            saidaGlobal << tempo/3600.0 << TAB;
            saidaGlobal << tempo/60.0 << TAB;
            saidaGlobal << tempo << TAB;
            saidaGlobal << tempInterna << TAB;
            saidaGlobal << tempPeleMedia << TAB;
            saidaGlobal << ambiente.TempOperativa() << endl;

            //saida dos dados espec�ficos de cada segmento
            for (IntPos i=0; i<lSegmentoSaida.size(); ++i) {
                *saidaTrans[i] << tempo/3600.0 << TAB;
                *saidaTrans[i] << tempo/60.0 << TAB;
                *saidaTrans[i] << tempo << TAB;
                lSegmentoSaida[i]->SaidaDadosArquivo(*saidaTrans[i]);
                *saidaTrans[i] << endl;
                //saidaTrans[i]->close();
            }
            tempoProximaSaida += dTempoSaida;

            tempoTotOut += (time(NULL) - tempoOutput);

        }
    } //tempo

    time_t tempoFim = time(NULL);


    controladorDaSimulacao.terminaSimulacao();
    barreiraDefinicaoEscopoProximaIteracao.wait();
	sincronizaFimExecucao.wait();

	cout << "Tempo gasto na finalizacao:: " << time(NULL) - tempoFim << " s" << endl;
	cout << "Tempo gasto OUTPUT:: " << tempoTotOut << " s" << endl;



    saidaGlobal.close();
    for (IntPos i=0; i<lSegmento.size(); ++i) saidaTrans[i]->close();

    //cout << "calor perdido na respiracao: " << tronco.CalorRespiracao(ambiente.TempOperativa(),ambiente.UmidadeRel()) << endl;

    SaidaXY(0,"saida/temp_cabeca.txt",cabeca);
    SaidaXY(0,"saida/temp_tronco.txt",tronco);
    SaidaLinha("saida/temp_pescoco.txt",pescoco);
    SaidaLinha("saida/temp_bracoesq.txt",bracoEsq);
    SaidaPolar(0,"saida/temp_antebracoesq.txt",antebracoEsq);
    SaidaXY(0,"saida/temp_maoesq.txt",maoEsq);
    SaidaLinha("saida/temp_coxaesq.txt",coxaEsq);
    SaidaPolar(0,"saida/temp_pernaesq.txt",pernaEsq);
    SaidaXY(0,"saida/temp_peesq.txt",peEsq);
    
    //system("gnuplot graf/temperatura.gnuplot");
    //system("evince temp*.eps &");
    
    //Vetor com endereco dos segmentos
    vector<Segmento*> lSegRed;
    lSegRed.push_back(&cabeca);
    lSegRed.push_back(&pescoco);
    lSegRed.push_back(&tronco);
    lSegRed.push_back(&bracoEsq);
    lSegRed.push_back(&antebracoEsq);
    lSegRed.push_back(&maoEsq);
    lSegRed.push_back(&coxaEsq);
    lSegRed.push_back(&pernaEsq);
    lSegRed.push_back(&peEsq);

    ofstream saidaTemp("saida/saida_temp.txt");

    saidaTemp << "#seg\tTar\tTve\tTpele\tTgord\tTmusc\tTosso\tTcer\tTpulm\tTcor\tTvisc\n";
    for (IntPos i=0; i<lSegRed.size(); ++i) {
        saidaTemp << i << "\t";
        saidaTemp << lSegRed[i]->TempArterial() << "\t";
        saidaTemp << lSegRed[i]->TempVenoso() << "\t";
        saidaTemp << lSegRed[i]->TempCompTecido(PELE) << "\t";
        saidaTemp << lSegRed[i]->TempCompTecido(GORDURA) << "\t";
        saidaTemp << lSegRed[i]->TempCompTecido(MUSCULO) << "\t";
        saidaTemp << lSegRed[i]->TempCompTecido(OSSO) << "\t";
        if (i==0) {
            saidaTemp << cabeca.TempCompTecido(CEREBRO) << "\t";
            saidaTemp << tronco.TempCompTecido(PULMAO) << "\t";
            saidaTemp << tronco.TempCompTecido(CORACAO) << "\t";
            saidaTemp << tronco.TempCompTecido(VISCERA) << "\t";
        }
        saidaTemp << endl;
    }
    saidaTemp.close();

    cout << "T retal: " << tronco.TempCompTecido(VISCERA) << endl;
    cout << "T timpano: " << cabeca.TempCompTecido(CEREBRO) << endl;
    cout << "Overhead das threads: " << tempoGastoParaConstruirThreads << " s" << endl;
    //cout << "T pele media: " << tempPeleMedia << endl;

}
    

/***************************************************************
***************************************************************/
void Corpo::AjustaCalorRespiracao()
{
    double calResp = tronco.CalorRespiracao(ambiente.TempOperativa(),
           ambiente.UmidadeRel());
    double volMuscCabeca = cabeca.VolumeTecido(MUSCULO);
    double volMuscPescoco = pescoco.VolumeTecido(MUSCULO);
    double volPulmao = tronco.VolumeTecido(PULMAO);
    muscCabeca.gerCalor = muscBasal.gerCalor-calResp/(3.0*volMuscCabeca);
    muscPescoco.gerCalor = muscBasal.gerCalor-calResp/(3.0*volMuscPescoco);
    pulmao.gerCalor = 339.0-calResp/(3.0*volPulmao);
}

/***************************************************************
***************************************************************/
void Corpo::AjustaVazao()
{
    //extremidades
    maoEsq.SetVazSanguePos(0.0);
    maoEsq.SetVazSangueAnt(maoEsq.VazSanguePeqVas());
    maoDir.SetVazSanguePos(0.0);
    maoDir.SetVazSangueAnt(maoDir.VazSanguePeqVas());

/*    if ( maoEsq.VazSangueAnt() != maoDir.VazSangueAnt() )
    {
    	cout << "DIFFF PO!!!!! :: " << maoEsq.VazSangueAnt() << maoDir.VazSangueAnt() << endl;
    	exit(1);
    }
*/

    peEsq.SetVazSanguePos(0.0);
    peEsq.SetVazSangueAnt(peEsq.VazSanguePeqVas());
    peDir.SetVazSanguePos(0.0);
    peDir.SetVazSangueAnt(peDir.VazSanguePeqVas());
    
    cabeca.SetVazSanguePos(0.0);
    cabeca.SetVazSangueAnt(cabeca.VazSanguePeqVas());

    //intermediarios
    antebracoEsq.SetVazSanguePos(maoEsq.VazSangueAnt());
    antebracoEsq.SetVazSangueAnt(antebracoEsq.VazSanguePos()
            +antebracoEsq.VazSanguePeqVas());
    antebracoDir.SetVazSanguePos(maoDir.VazSangueAnt());
    antebracoDir.SetVazSangueAnt(antebracoDir.VazSanguePos()
            +antebracoDir.VazSanguePeqVas());

    bracoEsq.SetVazSanguePos(antebracoEsq.VazSangueAnt());
    bracoEsq.SetVazSangueAnt(bracoEsq.VazSanguePos()
            +bracoEsq.VazSanguePeqVas());
    bracoDir.SetVazSanguePos(antebracoDir.VazSangueAnt());
    bracoDir.SetVazSangueAnt(bracoDir.VazSanguePos()
            +bracoDir.VazSanguePeqVas());

    pernaEsq.SetVazSanguePos(peEsq.VazSangueAnt());
    pernaEsq.SetVazSangueAnt(pernaEsq.VazSanguePos()
            +pernaEsq.VazSanguePeqVas());
   pernaDir.SetVazSanguePos(peDir.VazSangueAnt());
    pernaDir.SetVazSangueAnt(pernaDir.VazSanguePos()
            +pernaDir.VazSanguePeqVas());

    coxaEsq.SetVazSanguePos(pernaEsq.VazSangueAnt());
    coxaEsq.SetVazSangueAnt(coxaEsq.VazSanguePos()
            +coxaEsq.VazSanguePeqVas());
    coxaDir.SetVazSanguePos(pernaDir.VazSangueAnt());
    coxaDir.SetVazSangueAnt(coxaDir.VazSanguePos()
            +coxaDir.VazSanguePeqVas());

    pescoco.SetVazSanguePos(cabeca.VazSangueAnt());
    pescoco.SetVazSangueAnt(pescoco.VazSanguePos()
            +pescoco.VazSanguePeqVas());

    //tronco
    tronco.SetVazSanguePos(coxaEsq.VazSangueAnt()
            +coxaDir.VazSangueAnt()
            +bracoEsq.VazSangueAnt()
            +bracoDir.VazSangueAnt()
            +pescoco.VazSangueAnt());
    tronco.SetVazSangueAnt(tronco.VazSanguePos()
            +tronco.VazSanguePeqVas());

    pulmao.vazSangue = tronco.VazSangueAnt()/2481.0e-6;
    //mudar para volume do pulm�o calculado
}   

/***************************************************************
***************************************************************/
void Corpo::AjustaTemperatura()
{
    //extremidades
    maoEsq.SetTempArtEnt(antebracoEsq.TempArterial());
    maoEsq.SetTempVenEnt(999999.9999);
    maoDir.SetTempArtEnt(antebracoDir.TempArterial());
    maoDir.SetTempVenEnt(999999.9999);

    peEsq.SetTempArtEnt(pernaEsq.TempArterial());
    peEsq.SetTempVenEnt(999999.9999);
    peDir.SetTempArtEnt(pernaDir.TempArterial());
    peDir.SetTempVenEnt(999999.9999);

    cabeca.SetTempArtEnt(pescoco.TempArterial());
    cabeca.SetTempVenEnt(999999.9999);

    //intermediarios
    antebracoEsq.SetTempArtEnt(bracoEsq.TempArterial());
    antebracoEsq.SetTempVenEnt(maoEsq.TempVenoso());
    antebracoDir.SetTempArtEnt(bracoDir.TempArterial());
    antebracoDir.SetTempVenEnt(maoDir.TempVenoso());

    bracoEsq.SetTempArtEnt(tronco.TempArterial());
    bracoEsq.SetTempVenEnt(antebracoEsq.TempVenoso());
   bracoDir.SetTempArtEnt(tronco.TempArterial());
    bracoDir.SetTempVenEnt(antebracoDir.TempVenoso());

    pernaEsq.SetTempArtEnt(coxaEsq.TempArterial());
    pernaEsq.SetTempVenEnt(peEsq.TempVenoso());
    pernaDir.SetTempArtEnt(coxaDir.TempArterial());
    pernaDir.SetTempVenEnt(peDir.TempVenoso());

    coxaEsq.SetTempArtEnt(tronco.TempArterial());
    coxaEsq.SetTempVenEnt(pernaEsq.TempVenoso());
    coxaDir.SetTempArtEnt(tronco.TempArterial());
    coxaDir.SetTempVenEnt(pernaDir.TempVenoso());

    pescoco.SetTempArtEnt(tronco.TempArterial());
    pescoco.SetTempVenEnt(cabeca.TempVenoso());

    tronco.SetTempVenEnt((bracoEsq.TempVenoso()*bracoEsq.VazSangueAnt()
            +bracoDir.TempVenoso()*bracoDir.VazSangueAnt()
            +coxaEsq.TempVenoso()*coxaEsq.VazSangueAnt()
            +coxaDir.TempVenoso()*coxaDir.VazSangueAnt()
            +pescoco.TempVenoso()*pescoco.VazSangueAnt())/
            (bracoEsq.VazSangueAnt()
             +bracoDir.VazSangueAnt()
             +coxaEsq.VazSangueAnt()
             +coxaDir.VazSangueAnt()
             +pescoco.VazSangueAnt()));

}

/***************************************************************
***************************************************************/
void Corpo::AjustaConcentracao()
{
    if (regimePermanente) {
        //extremidades
        maoEsq.SetConcArtEnt(tronco.ConcArterial());
        maoDir.SetConcArtEnt(tronco.ConcArterial());
        peEsq.SetConcArtEnt(tronco.ConcArterial());
        peDir.SetConcArtEnt(tronco.ConcArterial());
        cabeca.SetConcArtEnt(tronco.ConcArterial());

        //intermediarios
        antebracoEsq.SetConcArtEnt(tronco.ConcArterial());
        antebracoDir.SetConcArtEnt(tronco.ConcArterial());
        bracoEsq.SetConcArtEnt(tronco.ConcArterial());
        bracoDir.SetConcArtEnt(tronco.ConcArterial());
        pernaEsq.SetConcArtEnt(tronco.ConcArterial());
        pernaDir.SetConcArtEnt(tronco.ConcArterial());
        coxaEsq.SetConcArtEnt(tronco.ConcArterial());
        coxaDir.SetConcArtEnt(tronco.ConcArterial());
        pescoco.SetConcArtEnt(tronco.ConcArterial());
    }
    else /*if (!regimePermanente)*/ {
        //extremidades
        maoEsq.SetConcArtEnt(antebracoEsq.ConcArterial());
        maoDir.SetConcArtEnt(antebracoDir.ConcArterial());
        peEsq.SetConcArtEnt(pernaEsq.ConcArterial());
        peDir.SetConcArtEnt(pernaDir.ConcArterial());
        cabeca.SetConcArtEnt(pescoco.ConcArterial());

        //intermediarios
        antebracoEsq.SetConcArtEnt(bracoEsq.ConcArterial());
        antebracoDir.SetConcArtEnt(bracoDir.ConcArterial());
        bracoEsq.SetConcArtEnt(tronco.ConcArterial());
        bracoDir.SetConcArtEnt(tronco.ConcArterial());
        pernaEsq.SetConcArtEnt(coxaEsq.ConcArterial());
        pernaDir.SetConcArtEnt(coxaDir.ConcArterial());
        coxaEsq.SetConcArtEnt(tronco.ConcArterial());
        coxaDir.SetConcArtEnt(tronco.ConcArterial());
        pescoco.SetConcArtEnt(tronco.ConcArterial());
    }

    //extremidades
    Gas<double> concZero(0.0,0.0);
    maoEsq.SetConcVenEnt(concZero);
    maoDir.SetConcVenEnt(concZero);
    peEsq.SetConcVenEnt(concZero);
    peDir.SetConcVenEnt(concZero);
    cabeca.SetConcVenEnt(concZero);

    //intermediarios
    antebracoEsq.SetConcVenEnt(maoEsq.ConcVenoso());
    antebracoDir.SetConcVenEnt(maoDir.ConcVenoso());
    bracoEsq.SetConcVenEnt(antebracoEsq.ConcVenoso());
    bracoDir.SetConcVenEnt(antebracoDir.ConcVenoso());
    pernaEsq.SetConcVenEnt(peEsq.ConcVenoso());
    pernaDir.SetConcVenEnt(peDir.ConcVenoso());
    coxaEsq.SetConcVenEnt(pernaEsq.ConcVenoso());
    coxaDir.SetConcVenEnt(pernaDir.ConcVenoso());
    pescoco.SetConcVenEnt(cabeca.ConcVenoso());

    Gas<double> concAux;
    for (IntPos ig=0; ig<NUM_GAS; ++ig) {
        concAux(ig) = (bracoEsq.ConcVenoso(ig)*bracoEsq.VazSangueAnt()
            +bracoDir.ConcVenoso(ig)*bracoDir.VazSangueAnt()
            +coxaEsq.ConcVenoso(ig)*coxaEsq.VazSangueAnt()
            +coxaDir.ConcVenoso(ig)*coxaDir.VazSangueAnt()
            +pescoco.ConcVenoso(ig)*pescoco.VazSangueAnt())/
            (bracoEsq.VazSangueAnt()
             +bracoDir.VazSangueAnt()
             +coxaEsq.VazSangueAnt()
             +coxaDir.VazSangueAnt()
             +pescoco.VazSangueAnt());
    }
    tronco.SetConcVenEnt(concAux);
}

/***************************************************************
***************************************************************/
TipoTecido Corpo::RetornaTipoTecido(IntPos num)
{
    //if (num==0) return PELE;
    switch (num) {
        case 0: return PELE; break;
        case 1: return GORDURA; break;
        case 2: return MUSCULO; break;
        case 3: return OSSO; break;
        case 4: return CEREBRO; break;
        case 5: return PULMAO; break;
        case 6: return CORACAO; break;
        case 7: return VISCERA; break;
        default: return PELE; break;
    }
}
   
/*****************************************************************************
* Calcula temperatura m�dia dos tecidos
*****************************************************************************/
void Corpo::CalcTempMediaTecido()
{
    vector<double> vol(8,0.0), temp(8,0.0);
    //enum TipoTecido {PELE-0,GORDURA-1,MUSCULO-2,OSSO-3,CEREBRO-4,PULMAO-5,CORACAO-6,VISCERA-7};

    double volAux=0.0;
    for (IntPos i=0; i<lSegmento.size(); ++i) {
        for (IntPos j=0; j<8; ++j) {
            volAux = lSegmento[i]->VolumeTecido(RetornaTipoTecido(j));
            vol[j]+=volAux;
            temp[j]+= lSegmento[i]->TempCompTecido(RetornaTipoTecido(j))*volAux;
        }
    }
    cout << "temperatura media dos tecidos" << endl;
    for (IntPos j=0; j<8; ++j) {
        temp[j]/=vol[j];
        cout << j << " " << temp[j] << endl;
    }

}
double Corpo::Calafrio(double tempPele, double tempInterna, double calorSup)
{
    double volMusculo = 26330.0e-6;
    double dTempPele = 34.1 - tempPele;
    double dTempInterna = 36.6 - tempInterna;
    double dCalorSup = calorSup - 69.62;
    if (dTempPele < 0.0) dTempPele = 0.0;
    if (dTempInterna < 0.0) dTempInterna = 0.0;
    if (dCalorSup < 0.0) dCalorSup = 0.0;
    
    double cte1 = 250.0,
           cte2 = 40.0,
           cte3 = 0.06;

    double gerMusculo =  (cte1*dTempInterna + cte2*dTempPele + cte3*dCalorSup)
        /volMusculo;

    if (gerMusculo < 0.0) gerMusculo = 0.0; 
    //if (gerMusculo > 429.0) gerMusculo = 429.0; 
    if (gerMusculo > 13200.0) gerMusculo = 13200.0; 
    return gerMusculo;
}

/*double Corpo::Sudorese(double tempPeleMedia, double tempPeleLocal, double tempInterna)
{
    double dTempPele = tempPeleMedia - 34.1;
    double dTempPeleLocal = tempPeleLocal - 34.1;
    double dTempInterna = tempInterna - 36.6;

    if (dTempPele < 0.0) dTempPele = 0.0;
    if (dTempPeleLocal < 0.0) dTempPeleLocal = 0.0;
    if (dTempInterna < 0.0) dTempInterna = 0.0;
    
    double cte1 = 100.0,
           cte2 = 11.0;

    return (cte1*dTempInterna + cte2*dTempPele)*exp(dTempPeleLocal/10.0);

}*/

double Corpo::VazaoPele(double tempPele, double tempInterna)
{   
    double dTempPele = tempPele- 34.1;
    double dTempInterna = tempInterna - 36.6;
    //if (dTempPele < 0.0) dTempPele = 0.0;
    //if (dTempInterna < 0.0) dTempInterna = 0.0;

    double cte1 = 1810.0e-6,
           cte2 = 181.0e-6;

    double vazao = cte1*dTempInterna + cte2*dTempPele;

    if (vazao > 3620.0e-6) vazao = 3620.0e-6;
    if (vazao < 90.5e-6) vazao = 90.5e-6;

    return vazao;

}

/***************************************************************
***************************************************************/
void Corpo::SaidaPolar(int iz, const char* nome, Segmento& segmento)
{
    ofstream saidaT(nome);
    for (IntPos j=0; j<segmento.CelulaVet()->Dim2(); j++) {
        saidaT << 0.0 << "\t";
        saidaT << segmento.Celulas(0,j,iz)->Centro().C2() << "\t";
        saidaT << segmento.Celulas(0,j,iz)->Temperatura() << endl;
        for (IntPos i=0; i<segmento.CelulaVet()->Dim1(); i++) {
            saidaT << segmento.Celulas(i,j,iz)->Centro().C1() << "\t";
            saidaT << segmento.Celulas(i,j,iz)->Centro().C2() << "\t";
            saidaT << segmento.Celulas(i,j,iz)->Temperatura() << endl;
        }
        saidaT << endl;
    }
    saidaT << 0.0 << "\t";
    saidaT << 2.0*M_PI+segmento.Celulas(0,0,iz)->Centro().C2() << "\t";
    saidaT << segmento.Celulas(0,0,iz)->Temperatura() << endl;
    for (IntPos i=0; i<segmento.CelulaVet()->Dim1(); i++) {
        saidaT << segmento.Celulas(i,0,iz)->Centro().C1() << "\t";
        saidaT << 2.0*M_PI+segmento.Celulas(i,0,iz)->Centro().C2() << "\t";
        saidaT << segmento.Celulas(i,0,iz)->Temperatura() << endl;
    }
    saidaT.close();
}

/***************************************************************
***************************************************************/
void Corpo::SaidaXZ(int iz, const char* nome, Segmento& segmento)
{
    ofstream saidaT(nome);
    for (IntPos j=0; j<segmento.CelulaVet()->Dim3(); j++) {
        for (IntPos i=0; i<segmento.CelulaVet()->Dim1(); i++) {
            saidaT << segmento.Celulas(i,iz,j)->Centro().C1() << "\t";
            saidaT << segmento.Celulas(i,iz,j)->Centro().C3() << "\t";
            saidaT << segmento.Celulas(i,iz,j)->Temperatura() << endl;
        }
        saidaT << endl;
    }
    saidaT.close();
}

/***************************************************************
***************************************************************/
void Corpo::SaidaXY(int iz, const char* nome, Segmento& segmento)
{
    ofstream saidaT(nome);
    for (IntPos j=0; j<segmento.CelulaVet()->Dim2(); j++) {
        for (IntPos i=0; i<segmento.CelulaVet()->Dim1(); i++) {
            saidaT << segmento.Celulas(i,j,iz)->Centro().C1() << "\t";
            saidaT << segmento.Celulas(i,j,iz)->Centro().C2() << "\t";
            saidaT << segmento.Celulas(i,j,iz)->Temperatura() << endl;
        }
        saidaT << endl;
    }
    saidaT.close();
}

/***************************************************************
***************************************************************/
void Corpo::SaidaLinha(const char* nome, Segmento& segmento)
{
    ofstream saidaT(nome);
    for (IntPos i=0; i<segmento.CelulaVet()->Dim1(); i++) {
        saidaT << segmento.Celulas(i,0,0)->Centro().C1() << "\t";
        saidaT << segmento.Celulas(i,0,0)->Temperatura() << endl;
    }
    saidaT << endl;
    saidaT.close();
}

/***************************************************************
***************************************************************/
/*void Corpo::SaidaCompTec(int iz, const char* nome, Segmento& segmento)
{
    ofstream saidaT(nome);
    for (IntPos j=0; j<segmento.CelulaVet()->Dim2(); j++) {
        saidaT << 0.0 << "\t";
        saidaT << segmento.Celulas(0,j,iz)->Centro().C2() << "\t";
        saidaT << segmento.Celulas(0,j,iz)->NumCompCel() << endl;
        for (IntPos i=0; i<segmento.CelulaVet()->Dim1(); i++) {
            saidaT << segmento.Celulas(i,j,iz)->Centro().C1() << "\t";
            saidaT << segmento.Celulas(i,j,iz)->Centro().C2() << "\t";
            saidaT << segmento.Celulas(i,j,iz)->NumCompCel() << endl;
        }
        saidaT << endl;
    }
    saidaT << 0.0 << "\t";
    saidaT << 2.0*M_PI+segmento.Celulas(0,0,iz)->Centro().C2() << "\t";
    saidaT << segmento.Celulas(0,0,iz)->NumCompCel() << endl;
    for (IntPos i=0; i<segmento.CelulaVet()->Dim1(); i++) {
        saidaT << segmento.Celulas(i,0,iz)->Centro().C1() << "\t";
        saidaT << 2.0*M_PI+segmento.Celulas(i,0,iz)->Centro().C2() << "\t";
        saidaT << segmento.Celulas(i,0,iz)->NumCompCel() << endl;
    }
    saidaT.close();
}*/
