// Funcoes membro da classe ListaFace

#include "listaface.h"

/*****************************************************************
* Construtor
******************************************************************/
ListaFace::ListaFace()
{
}

/*****************************************************************
* Destrutor
******************************************************************/
ListaFace::~ListaFace()
{
    for (IntPos i=0; i<faces.size(); ++i) delete faces[i];
    faces.clear();
}

//-----------------------------------------------------------------------------
void ListaFace::GeraListaFace(IntPos nDim, SistCoord sCoord,
                              ListaCelula& lCel, Ponto<double> tamExt)
{
    tamanhoExterno = tamExt;
    sistCoord = sCoord;

    //Numero de faces
    IntPos nFace = lCel.CalcNumFace(nDim,sistCoord);
    faces.resize(nFace);

    //Geracao das faces 
    IntPos iFace = 0;
    VetInt iCel(nDim,0);
    //Cria as faces enquanto varre as celulas
    for (iCel[0]=0; iCel[0]<lCel.Dim(0); ++iCel[0]) {
        if (nDim==1) CriaFace(nDim,lCel,iFace,iCel);
        else {
            for (iCel[1]=0; iCel[1]<lCel.Dim(1); ++iCel[1]) {
                if (nDim==2) CriaFace(nDim,lCel,iFace,iCel);
                else {
                    for (iCel[2]=0; iCel[2]<lCel.Dim(2); ++iCel[2]) {
                        CriaFace(nDim,lCel,iFace,iCel);
                    }
                }
            }
        }
    }

    //Condutividade termica nas faces
    for (IntPos i=0; i<nFace; ++i) faces[i]->CalcCondTerm();
}

/***************************************************************** 
* Verifica quais as faces devem ser criadas para uma celula.
*****************************************************************/ 
void ListaFace::CriaFace(IntPos nDim, ListaCelula& lCel,
                        IntPos& iF, VetInt iC)
{
    if (sistCoord==CARTESIANO) {
        if (lCel.Ini(iC,0)) CriaFaceExt(iF,iC,W,WE,lCel);       //externa W
        if (lCel.Fim(iC,0)) CriaFaceExt(iF,iC,E,WE,lCel);       //externa E
        else CriaFaceInt(iF,iC,WE,lCel);                        //interna E
        if (nDim>=2) {
            if (lCel.Ini(iC,1)) CriaFaceExt(iF,iC,S,SN,lCel);   //externa S
            if (lCel.Fim(iC,1)) CriaFaceExt(iF,iC,N,SN,lCel);   //externa N
            else CriaFaceInt(iF,iC,SN,lCel);                    //interna N
        }
        if (nDim==3) {
            if (lCel.Ini(iC,2)) CriaFaceExt(iF,iC,F,FB,lCel);   //externa F
            if (lCel.Fim(iC,2)) CriaFaceExt(iF,iC,B,FB,lCel);   //externa B
            else CriaFaceInt(iF,iC,FB,lCel);                    //interna B
        }
    }
    if (sistCoord==CILINDRICO) {
        if (nDim==1) {
            if (lCel.Ini(iC,0)) CriaFaceExt(iF,iC,W,WE,lCel);    //externa W
        }
        if (lCel.Fim(iC,0)) CriaFaceExt(iF,iC,E,WE,lCel);        //externa E
        else CriaFaceInt(iF,iC,WE,lCel);                         //interna E
        if (nDim>=2) {
            CriaFaceInt(iF,iC,SN,lCel);                          //interna N
        }
        if (nDim==3) {
            if (lCel.Ini(iC,2)) CriaFaceExt(iF,iC,F,FB,lCel);    //externa F
            if (lCel.Fim(iC,2)) CriaFaceExt(iF,iC,B,FB,lCel);    //externa B
            else CriaFaceInt(iF,iC,FB,lCel);                     //interna B
        }

    }
    if (sistCoord==CIL_RZ) {
        if (lCel.Ini(iC,0)) CriaFaceExt(iF,iC,W,WE,lCel);        //externa W
        if (lCel.Fim(iC,0)) CriaFaceExt(iF,iC,E,WE,lCel);        //externa E
        else CriaFaceInt(iF,iC,WE,lCel);                         //interna E
        if (nDim>=2) {
            if (lCel.Ini(iC,1)) CriaFaceExt(iF,iC,S,SN,lCel);    //externa S
            if (lCel.Fim(iC,1)) CriaFaceExt(iF,iC,N,SN,lCel);    //externa N
            else CriaFaceInt(iF,iC,SN,lCel);                     //interna N
        }
    }
}

//-----------------------------------------------------------------------------
//jF - posicao da face interna posterior a celula
//coord - coordenada que esta o vizinho
//cilindrico e esferico, quando teta=2 PI, face ocorre entre primeira e ultima celula
//para isso pode acrescentar flag fechaCirculo, no caso de true a proxima celula vizinha
//sera a primeira, no caso de false a proxima celula vizinha sera a proxima da matriz.
void ListaFace::AtualizaVizinhoInt(IntPos jF, VetInt celAnt, Direcao vDir, ListaCelula& lCel)
{
    VetInt celPos = lCel.AchaCelPos(sistCoord,celAnt,vDir);

    faces[jF]->SetCelViz1(lCel.Em(celAnt));
    faces[jF]->SetCelViz2(lCel.Em(celPos));
    if (vDir==WE) {
        lCel.Em(celAnt)->NovoVizinho(lCel.Em(celPos),faces[jF],E);  
        lCel.Em(celPos)->NovoVizinho(lCel.Em(celAnt),faces[jF],W);
    }
    else if (vDir==SN) {
        lCel.Em(celAnt)->NovoVizinho(lCel.Em(celPos),faces[jF],N);  
        lCel.Em(celPos)->NovoVizinho(lCel.Em(celAnt),faces[jF],S);
    }
    else /*if (vDir==FB)*/ {
        lCel.Em(celAnt)->NovoVizinho(lCel.Em(celPos),faces[jF],B);  
        lCel.Em(celPos)->NovoVizinho(lCel.Em(celAnt),faces[jF],F);
    }
}

//-----------------------------------------------------------------------------
//jF - posicao da face externa
//jC - posicao da celula
//sent - sentido que esta a face em relacao a celula
void ListaFace::AtualizaVizinhoExt(IntPos jF, VetInt jC, Sentido sent, ListaCelula& lCel)
{
    faces[jF]->SetCelViz1(lCel.Em(jC));
    lCel.Em(jC)->NovoVizinho(0,faces[jF],sent); 
}

//-----------------------------------------------------------------------------
void ListaFace::CriaFaceInt(IntPos& kF, VetInt kC, Direcao vDir, ListaCelula& lCel)
{
    faces[kF] = new FaceInterna(vDir);
    AtualizaVizinhoInt(kF,kC,vDir,lCel);
    //Centro
    faces[kF]->SetCentro(faces[kF]->CelViz1()->Centro().C1(),
                         faces[kF]->CelViz1()->Centro().C2(), 
                         faces[kF]->CelViz1()->Centro().C3());
    faces[kF]->SetCentroCoord(vDir,faces[kF]->CelViz1()->Centro().PtCoord(vDir)
                              +faces[kF]->CelViz1()->Tamanho().PtCoord(vDir)/2.0);
    //Distancia entre centro das celulas vizinhas
    if (vDir==SN && sistCoord==CILINDRICO && lCel.Fim(kC,1))
        faces[kF]->SetDCelula(fabs(2.0*M_PI
                    +faces[kF]->CelViz2()->Centro().PtCoord(vDir)
                    -faces[kF]->CelViz1()->Centro().PtCoord(vDir)));
    else
        faces[kF]->SetDCelula(fabs(faces[kF]->CelViz2()->Centro().PtCoord(vDir)
                    -faces[kF]->CelViz1()->Centro().PtCoord(vDir)));
    kF++;
}
//-----------------------------------------------------------------------------
void ListaFace::CriaFaceExt(IntPos& kF, VetInt kC, Sentido vSent, Direcao vDir, ListaCelula& lCel)
{
    faces[kF] = new FaceExterna(vSent);
    //faces[kF]->SetFronteira(vPar);
    AtualizaVizinhoExt(kF,kC,vSent,lCel);
    //Centro
    faces[kF]->SetCentro(faces[kF]->CelViz1()->Centro().C1(),
                         faces[kF]->CelViz1()->Centro().C2(),
                         faces[kF]->CelViz1()->Centro().C3());
    if (vSent==W||vSent==S||vSent==F) faces[kF]->SetCentroCoord(vDir, 0.0);
    else faces[kF]->SetCentroCoord(vDir, tamanhoExterno.PtCoord(vDir));
    //Distancia entre face e centro da celula
    faces[kF]->SetDCelula(fabs(faces[kF]->CelViz1()->Centro().PtCoord(vDir)
                          -faces[kF]->Centro().PtCoord(vDir)));
    kF++;
}
